@@ -6,8 +6,7 @@ use mongodb::bson::{self, Document};
 use serde::{Deserialize, Serialize};
 use std::{convert::TryFrom, fmt::Display, str::FromStr};
 
-#[derive(Serialize, Deserialize, Debug, Clone, PartialEq)]
-#[serde(tag = "t", content = "c")]
+#[derive(Debug, Clone, PartialEq)]
 pub enum Data {
     Bool(bool),
     U64(u64),
