@@ -3,7 +3,7 @@ use std::{
     fmt::{self, Display, Formatter},
 };
 
-use crate::{CryptoError, Entry, EntryPath, HasBuilder, States, Storer};
+use crate::{CryptoError, Entry, EntryPath, HasBuilder, State, Storer};
 use async_trait::async_trait;
 use futures::StreamExt;
 use mongodb::{
@@ -162,7 +162,7 @@ impl Storer for MongoStorer {
         }
     }
 
-    async fn create(&self, path: EntryPath, value: States) -> Result<bool, CryptoError> {
+    async fn create(&self, path: EntryPath, value: State) -> Result<bool, CryptoError> {
         let filter = bson::doc! { "path": &path };
         let entry = Entry { path, value };
         let filter_options = mongodb::options::ReplaceOptions::builder()
